"""
About
=====

Example program for listing folders and getting a dashboard of a remote Grafana
instance. By default, it uses `play.grafana.org`. Please adjust to your needs.


Synopsis
========
::

    source .venv/bin/activate
    python examples/folders-dashboard.py | jq
"""
import json
import sys

from grafana_client import GrafanaApi


def main():

    # Connect to public Grafana instance of Grafana Labs fame.
    grafana = GrafanaApi(host="play.grafana.org")

    print("## All folders on play.grafana.org", file=sys.stderr)
    folders = grafana.folder.get_all_folders()
    print(json.dumps(folders, indent=2))

    print("## Dashboard with UID 000000012 at play.grafana.org", file=sys.stderr)
    dashboard_000000012 = grafana.dashboard.get_dashboard("000000012")
    print(json.dumps(dashboard_000000012, indent=2))


if __name__ == "__main__":
    main()



"""
About
=====

Example program for submitting a query to a data source.

While the ``--query`` option is obligatory, the ``--store`` option is not.
It will be used to designate the ``database`` name, with e.g. InfluxDB,
or the ``metric`` name, with e.g. Prometheus.

Synopsis
========
::

    # Query the InfluxDB 1.x data source on `play.grafana.org`.
    python examples/datasource-smartquery.py --uid=000000002 --query="SHOW RETENTION POLICIES on _internal"
    python examples/datasource-smartquery.py --uid=000000002 --store=site --query="SHOW FIELD KEYS"

    # Query the Prometheus data source on `play.grafana.org`.
    # FIXME: Does not work yet.
    python \
        examples/datasource-smartquery.py --uid=000000008 --store=node_boot_time \
        --query='time() - node_boot_time_seconds{job="node", instance=~"demo.do.prometheus.io:.*"}'

"""
import json
import logging
from optparse import OptionParser

import requests

from grafana_client import GrafanaApi
from grafana_client.model import DatasourceIdentifier
from grafana_client.util import setup_logging

logger = logging.getLogger(__name__)


def run(grafana: GrafanaApi):
    parser = OptionParser()
    parser.add_option("--uid", dest="uid", help="Data source UID")
    parser.add_option("--store", dest="store", help="Database or metric to be addressed")
    parser.add_option("--query", dest="query", help="Query expression")
    (options, args) = parser.parse_args()
    if not options.uid:
        parser.error("Option --uid required")
    if not options.query:
        parser.error("Option --query required")

    # Query the data source by UID.
    response = grafana.datasource.smartquery(
        DatasourceIdentifier(uid=options.uid), expression=options.query, store=options.store
    )

    # Display the outcome in JSON format.
    print(json.dumps(response, indent=2))


if __name__ == "__main__":

    setup_logging(level=logging.DEBUG)

    # Connect to Grafana instance and run health check.
    grafana_client = GrafanaApi.from_url("https://play.grafana.org/")

    try:
        grafana_client.connect()
    except requests.exceptions.ConnectionError:
        logger.exception("Connecting to Grafana failed")
        raise SystemExit(1)

    run(grafana_client)



from .base import Base


class Rbac(Base):
    def __init__(self, client):
        super(Rbac, self).__init__(client)
        self.client = client

    def get_rbac_roles_all(self):
        """
        The Rbac is only available in Grafana Enterprise.

        :return:
        """
        roles_path = "/access-control/roles"
        r = self.client.GET(roles_path)
        return r

    def add_rbac_role_team(self, team_id, role_uid):
        """
        The Rbac is only available in Grafana Enterprise.

        :param team_id:
        :param role_uid:
        :return:
        """
        role_team_path = "/access-control/teams/%s/roles" % team_id
        r = self.client.POST(role_team_path, json={"roleUid": role_uid})
        return r

    def add_rbac_roles_team(self, team_id, role_uids):
        """
        The Rbac is only available in Grafana Enterprise.

        :param team_id:
        :param role_uids:
        :return:
        """
        role_team_path = "/access-control/teams/%s/roles" % team_id
        r = self.client.PUT(role_team_path, json={"roleUids": role_uids})
        return r

    def remove_rbac_role_team(self, team_id, role_uid):
        """
        The Rbac is only available in Grafana Enterprise.

        :param team_id:
        :param role_uid:
        :return:
        """
        role_team_path = "/access-control/teams/%s/roles/%s" % (team_id, role_uid)
        r = self.client.DELETE(role_team_path)
        return r



import unittest

import requests_mock

from grafana_client import GrafanaApi

ALERTRULE = {
    "id": 2,
    "uid": "bUUGqLiVk",
    "orgID": 1,
    "folderUID": "4",
    "ruleGroup": "Any",
    "title": "ApiAlerts",
    "condition": "B",
    "data": [],
    "updated": "2022-08-17T13:00:21Z",
    "noDataState": "NoData",
    "execErrState": "Alerting",
    "for": 300000000000,
    "labels": {"alertGroup": "any", "region": "eu-west-1"},
}


class AlertingProvisioningTestCase(unittest.TestCase):
    def setUp(self):
        self.grafana = GrafanaApi(("admin", "admin"), host="localhost", url_path_prefix="", protocol="http")

    @requests_mock.Mocker()
    def test_get_alertrule(self, m):
        m.get("http://localhost/api/v1/provisioning/alert-rules/bUUGqLiVk", json=ALERTRULE)
        response = self.grafana.alertingprovisioning.get_alertrule("bUUGqLiVk")
        self.assertEqual(response["uid"], "bUUGqLiVk")

    @requests_mock.Mocker()
    def test_delete_alertrule(self, m):
        m.delete("http://localhost/api/v1/provisioning/alert-rules/bUUGqLiVk", json={"uid": "bUUGqLiVk"})
        response = self.grafana.alertingprovisioning.delete_alertrule("bUUGqLiVk")
        self.assertEqual(response["uid"], "bUUGqLiVk")

    @requests_mock.Mocker()
    def test_create_alertrule_default(self, m):
        m.post(
            "http://localhost/api/v1/provisioning/alert-rules",
            json=ALERTRULE,
        )

        response = self.grafana.alertingprovisioning.create_alertrule(ALERTRULE)
        self.assertEqual(response["uid"], "bUUGqLiVk")

        # Verify request header.
        history = m.request_history
        headers = history[0].headers
        self.assertNotIn("X-Disable-Provenance", headers)

    @requests_mock.Mocker()
    def test_create_alertrule_disable_provenance(self, m):
        m.post(
            "http://localhost/api/v1/provisioning/alert-rules",
            json=ALERTRULE,
        )

        response = self.grafana.alertingprovisioning.create_alertrule(ALERTRULE, disable_provenance=True)
        self.assertEqual(response["uid"], "bUUGqLiVk")

        # Verify request header.
        history = m.request_history
        headers = history[0].headers
        self.assertIn("X-Disable-Provenance", headers)
        self.assertEqual(headers["X-Disable-Provenance"], "true")

    @requests_mock.Mocker()
    def test_update_alertrule_default(self, m):
        m.put(
            "http://localhost/api/v1/provisioning/alert-rules/bUUGqLiVk",
            json=ALERTRULE,
        )

        response = self.grafana.alertingprovisioning.update_alertrule(alertrule_uid="bUUGqLiVk", alertrule=ALERTRULE)
        self.assertEqual(response["uid"], "bUUGqLiVk")

        # Verify request header.
        history = m.request_history
        headers = history[0].headers
        self.assertNotIn("X-Disable-Provenance", headers)

    @requests_mock.Mocker()
    def test_update_alertrule_disable_provenance(self, m):
        m.put(
            "http://localhost/api/v1/provisioning/alert-rules/bUUGqLiVk",
            json=ALERTRULE,
        )

        response = self.grafana.alertingprovisioning.update_alertrule(
            alertrule_uid="bUUGqLiVk", alertrule=ALERTRULE, disable_provenance=True
        )
        self.assertEqual(response["uid"], "bUUGqLiVk")

        # Verify request header.
        history = m.request_history
        headers = history[0].headers
        self.assertIn("X-Disable-Provenance", headers)
        self.assertEqual(headers["X-Disable-Provenance"], "true")



import logging
import unittest

from grafana_client.util import as_bool, setup_logging


class UtilTestCase(unittest.TestCase):
    def test_setup_logging(self):
        setup_logging(level=logging.WARNING)

    def test_as_bool_success(self):
        self.assertTrue(as_bool("true"))
        self.assertFalse(as_bool("false"))
        self.assertFalse(as_bool(None))

    def test_as_bool_failure(self):
        self.assertRaises(ValueError, lambda: as_bool("foo"))



import sys
import unittest

if sys.version_info > (3, 0):
    from unittest.mock import Mock, patch
else:
    from mock import patch, Mock

import requests

from grafana_client.api import GrafanaApi
from grafana_client.client import GrafanaClientError, HeaderAuth, TokenAuth


class MockResponse:
    def __init__(self, status_code, headers=None, json_data=None):
        self.status_code = status_code
        self.headers = headers or {"Content-Type": "application/json"}
        self.json_data = json_data

    def json(self):
        return self.json_data


class TestGrafanaClient(unittest.TestCase):
    def test_grafana_client_user_agent_default(self):
        grafana = GrafanaApi.from_url()

        from grafana_client import __appname__, __version__

        user_agent = f"{__appname__}/{__version__}"

        self.assertTrue(user_agent.startswith("grafana-client"))
        self.assertEqual(grafana.client.s.headers["User-Agent"], user_agent)

    def test_grafana_client_user_agent_custom(self):
        grafana = GrafanaApi(
            ("admin", "admin"), host="localhost", url_path_prefix="", protocol="https", user_agent="foobar/3000"
        )
        self.assertEqual(grafana.client.s.headers["User-Agent"], "foobar/3000")

    @patch("grafana_client.client.GrafanaClient.__getattr__")
    def test_grafana_client(self, mock_get):
        mock_get.return_value = Mock()
        mock_get.return_value.return_value = """{
  "email": "user@mygraf.com",
  "name": "admin",
  "login": "admin",
  "theme": "light",
  "orgId": 1,
  "isGrafanaAdmin": true
}"""
        grafana = GrafanaApi(("admin", "admin"), host="localhost", url_path_prefix="", protocol="https")
        grafana.users.find_user("test@example.org")

    def test_grafana_client_no_verify(self):
        grafana = GrafanaApi(
            ("admin", "admin"),
            host="localhost",
            url_path_prefix="",
            protocol="https",
            verify=False,
        )
        grafana.client.s.get = Mock(name="get")
        grafana.client.s.get.return_value = MockResponse(
            status_code=200,
            json_data={
                "email": "user@mygraf.com",
                "name": "admin",
                "login": "admin",
                "theme": "light",
                "orgId": 1,
                "isGrafanaAdmin": True,
            },
        )

        basic_auth = requests.auth.HTTPBasicAuth("admin", "admin")
        grafana.users.find_user("test@example.org")
        grafana.client.s.get.assert_called_once_with(
            "https://localhost/api/users/lookup?loginOrEmail=test@example.org",
            auth=basic_auth,
            headers=None,
            json=None,
            data=None,
            verify=False,
            timeout=5.0,
        )

    def test_grafana_client_timeout(self):
        grafana = GrafanaApi(
            ("admin", "admin"),
            host="play.grafana.org",
            url_path_prefix="",
            protocol="https",
            verify=False,
            timeout=0.0001,
        )

        with self.assertRaises(requests.exceptions.Timeout):
            grafana.folder.get_all_folders()

    def test_grafana_client_basic_auth(self):
        grafana = GrafanaApi(("admin", "admin"), host="localhost", url_path_prefix="", protocol="https", port="3000")
        self.assertTrue(isinstance(grafana.client.auth, requests.auth.HTTPBasicAuth))

    def test_grafana_client_token_auth(self):
        grafana = GrafanaApi(
            "alongtoken012345etc",
            host="localhost",
            url_path_prefix="",
            protocol="https",
        )
        self.assertTrue(isinstance(grafana.client.auth, TokenAuth))

    def test_tokenauth(self):
        tokenauth = TokenAuth("VerySecretToken")
        request = requests.Request()
        tokenauth(request)
        self.assertEqual(request.headers["Authorization"], "Bearer VerySecretToken")

    def test_headerauth(self):
        headerauth = HeaderAuth(name="X-WEBAUTH-USER", value="foobar")
        request = requests.Request()
        headerauth(request)
        self.assertEqual(request.headers["X-WEBAUTH-USER"], "foobar")

    @patch("grafana_client.client.GrafanaClient.__getattr__")
    def test_grafana_client_connect_success(self, mock_get):
        payload = {"commit": "14e988bd22", "database": "ok", "version": "9.0.1"}
        mock_get.return_value = Mock()
        mock_get.return_value.return_value = payload
        grafana = GrafanaApi(auth=None, host="localhost", url_path_prefix="", protocol="http", port="3000")
        grafana_info = grafana.connect()
        self.assertEqual(grafana_info, payload)

    def test_grafana_client_connect_failure(self):
        grafana = GrafanaApi(auth=None, host="localhost", url_path_prefix="", protocol="http", port="32425")
        self.assertRaises(requests.exceptions.ConnectionError, lambda: grafana.connect())

    @patch("grafana_client.client.GrafanaClient.__getattr__")
    def test_grafana_client_version(self, mock_get):
        mock_get.return_value = Mock()
        mock_get.return_value.return_value = {"commit": "14e988bd22", "database": "ok", "version": "9.0.1"}
        grafana = GrafanaApi(auth=None, host="localhost", url_path_prefix="", protocol="http", port="3000")
        self.assertEqual(grafana.version, "9.0.1")

    def test_grafana_client_non_json_response(self):
        grafana = GrafanaApi.from_url("https://httpbin.org/html")
        self.assertRaises(GrafanaClientError, lambda: grafana.connect())

    def test_grafana_client_204_no_content_response(self):
        grafana = GrafanaApi.from_url()
        grafana.client.s.delete = Mock(name="get")
        grafana.client.s.delete.return_value = MockResponse(
            status_code=204,
        )
        response = grafana.alertingprovisioning.delete_alertrule("foobar")
        assert response is None



import os
import unittest
from unittest import mock

import requests

from grafana_client import GrafanaApi, HeaderAuth, TokenAuth


class TestGrafanaApiFactories(unittest.TestCase):
    """
    Test the two factory methods, `GrafanaApi.from_url()`, and
    `GrafanaApi.from_env()`, with different input parameters.
    """

    def test_from_url_default(self):
        grafana = GrafanaApi.from_url()
        self.assertIsInstance(grafana.client.auth, requests.auth.HTTPBasicAuth)
        self.assertEqual(grafana.client.auth.username, "admin")
        self.assertEqual(grafana.client.auth.password, "admin")
        self.assertEqual(grafana.client.url_host, "localhost")
        self.assertEqual(grafana.client.url_port, 3000)
        self.assertEqual(grafana.client.url_path_prefix, "")
        self.assertEqual(grafana.client.url_protocol, "http")
        self.assertEqual(grafana.client.verify, True)
        self.assertEqual(grafana.client.timeout, 5.0)

    def test_from_url_anonymous(self):
        grafana = GrafanaApi.from_url("http://localhost:3000")
        self.assertEqual(grafana.client.auth, None)

    def test_from_url_basicauth(self):
        grafana = GrafanaApi.from_url(credential=("foo", "bar"))
        self.assertIsInstance(grafana.client.auth, requests.auth.HTTPBasicAuth)
        self.assertEqual(grafana.client.auth.username, "foo")
        self.assertEqual(grafana.client.auth.password, "bar")

    def test_from_url_tokenauth(self):
        grafana = GrafanaApi.from_url(credential="VerySecretToken")
        self.assertIsInstance(grafana.client.auth, TokenAuth)
        self.assertEqual(grafana.client.auth.token, "VerySecretToken")

    def test_from_url_headerauth(self):
        grafana = GrafanaApi.from_url(credential=HeaderAuth(name="X-WEBAUTH-USER", value="foobar"))
        self.assertIsInstance(grafana.client.auth, HeaderAuth)
        self.assertEqual(grafana.client.auth.name, "X-WEBAUTH-USER")
        self.assertEqual(grafana.client.auth.value, "foobar")

    def test_from_url_auth_precedence(self):
        grafana = GrafanaApi.from_url("http://foo:bar@localhost:3000", credential="VerySecretToken")
        self.assertIsInstance(grafana.client.auth, TokenAuth)
        self.assertEqual(grafana.client.auth.token, "VerySecretToken")

    def test_from_url_auth_invalid(self):
        self.assertRaises(TypeError, lambda: GrafanaApi.from_url("http://foo:bar@localhost:3000", credential=42.42))

    def test_from_url_full_on(self):
        grafana = GrafanaApi.from_url("https://foo:bar@daq.example.org/grafana/?verify=false")
        self.assertIsInstance(grafana.client.auth, requests.auth.HTTPBasicAuth)
        self.assertEqual(grafana.client.auth.username, "foo")
        self.assertEqual(grafana.client.auth.password, "bar")
        self.assertEqual(grafana.client.url_host, "daq.example.org")
        self.assertEqual(grafana.client.url_port, None)
        self.assertEqual(grafana.client.url_path_prefix, "grafana/")
        self.assertEqual(grafana.client.url_protocol, "https")
        self.assertEqual(grafana.client.verify, False)
        self.assertEqual(grafana.client.timeout, 5.0)

    def test_from_env_default(self):
        grafana = GrafanaApi.from_env()
        self.assertIsInstance(grafana.client.auth, requests.auth.HTTPBasicAuth)
        self.assertEqual(grafana.client.auth.username, "admin")
        self.assertEqual(grafana.client.auth.password, "admin")
        self.assertEqual(grafana.client.url_host, "localhost")
        self.assertEqual(grafana.client.url_port, 3000)
        self.assertEqual(grafana.client.url_path_prefix, "")
        self.assertEqual(grafana.client.url_protocol, "http")
        self.assertEqual(grafana.client.verify, True)
        self.assertEqual(grafana.client.timeout, 5.0)

    @mock.patch.dict(os.environ, {"GRAFANA_URL": "http://localhost:3000"})
    def test_from_env_anonymous(self):
        grafana = GrafanaApi.from_env()
        self.assertEqual(grafana.client.auth, None)

    @mock.patch.dict(os.environ, {"GRAFANA_URL": "http://foo:bar@localhost:3000"})
    def test_from_env_basicauth(self):
        grafana = GrafanaApi.from_env()
        self.assertIsInstance(grafana.client.auth, requests.auth.HTTPBasicAuth)
        self.assertEqual(grafana.client.auth.username, "foo")
        self.assertEqual(grafana.client.auth.password, "bar")

    @mock.patch.dict(os.environ, {"GRAFANA_TOKEN": "VerySecretToken"})
    def test_from_env_tokenauth(self):
        grafana = GrafanaApi.from_env()
        self.assertIsInstance(grafana.client.auth, TokenAuth)
        self.assertEqual(grafana.client.auth.token, "VerySecretToken")

    @mock.patch.dict(os.environ, {"GRAFANA_URL": "http://foo:bar@localhost:3000", "GRAFANA_TOKEN": "VerySecretToken"})
    def test_from_env_auth_precedence(self):
        grafana = GrafanaApi.from_env()
        self.assertIsInstance(grafana.client.auth, TokenAuth)
        self.assertEqual(grafana.client.auth.token, "VerySecretToken")

    @mock.patch.dict(os.environ, {"GRAFANA_URL": "https://foo:bar@daq.example.org/grafana/?verify=false"})
    def test_from_env_full_on(self):
        grafana = GrafanaApi.from_env()
        self.assertIsInstance(grafana.client.auth, requests.auth.HTTPBasicAuth)
        self.assertEqual(grafana.client.auth.username, "foo")
        self.assertEqual(grafana.client.auth.password, "bar")
        self.assertEqual(grafana.client.url_host, "daq.example.org")
        self.assertEqual(grafana.client.url_port, None)
        self.assertEqual(grafana.client.url_path_prefix, "grafana/")
        self.assertEqual(grafana.client.url_protocol, "https")
        self.assertEqual(grafana.client.verify, False)
        self.assertEqual(grafana.client.timeout, 5.0)



import unittest
from test.elements.test_datasource_fixtures import (
    DATAFRAME_RESPONSE_HEALTH_PROMETHEUS,
    ELASTICSEARCH_DATASOURCE,
    INFLUXDB1_DATASOURCE,
    PERMISSION_DATASOURCE,
    PROMETHEUS_DATA_RESPONSE,
    PROMETHEUS_DATASOURCE,
)
from unittest.mock import Mock, patch

import requests_mock

from grafana_client import GrafanaApi
from grafana_client.client import (
    GrafanaBadInputError,
    GrafanaClientError,
    GrafanaServerError,
)
from grafana_client.model import DatasourceIdentifier


class DatasourceTestCase(unittest.TestCase):
    def setUp(self):
        self.grafana = GrafanaApi(("admin", "admin"), host="localhost", url_path_prefix="", protocol="http")

    @requests_mock.Mocker()
    def test_health(self, m):
        m.get(
            "http://localhost/api/datasources/uid/foobar/health",
            json={"message": "TwinMaker datasource successfully configured (For play.grafana.org)", "status": "OK"},
        )

        result = self.grafana.datasource.health("foobar")
        self.assertEqual(result["status"], "OK")

    @requests_mock.Mocker()
    def test_get_datasource_by_id(self, m):
        m.get(
            "http://localhost/api/datasources/42",
            json=PROMETHEUS_DATASOURCE,
        )

        result = self.grafana.datasource.get(DatasourceIdentifier(id="42"))
        self.assertEqual(result["type"], "prometheus")

    @requests_mock.Mocker()
    def test_get_datasource_by_uid(self, m):
        m.get(
            "http://localhost/api/datasources/uid/h8KkCLt7z",
            json=PROMETHEUS_DATASOURCE,
        )

        result = self.grafana.datasource.get(DatasourceIdentifier(uid="h8KkCLt7z"))
        self.assertEqual(result["type"], "prometheus")

    @requests_mock.Mocker()
    def test_get_datasource_by_name(self, m):
        m.get(
            "http://localhost/api/datasources/name/Prometheus",
            json=PROMETHEUS_DATASOURCE,
        )

        result = self.grafana.datasource.get(DatasourceIdentifier(name="Prometheus"))
        self.assertEqual(result["type"], "prometheus")

    @requests_mock.Mocker()
    def test_get_datasource_invalid(self, m):
        self.assertRaises(KeyError, lambda: self.grafana.datasource.get(DatasourceIdentifier()))

    @requests_mock.Mocker()
    def test_create_datasource(self, m):
        m.post(
            "http://localhost/api/datasources",
            json=PROMETHEUS_DATASOURCE,
        )

        result = self.grafana.datasource.create_datasource(PROMETHEUS_DATASOURCE)
        self.assertEqual(result["type"], "prometheus")

    @requests_mock.Mocker()
    def test_update_datasource(self, m):
        m.put(
            "http://localhost/api/datasources/42",
            json=PROMETHEUS_DATASOURCE,
        )

        result = self.grafana.datasource.update_datasource(42, PROMETHEUS_DATASOURCE)
        self.assertEqual(result["type"], "prometheus")

    @requests_mock.Mocker()
    def test_update_datasource_by_uid(self, m):
        m.put(
            "http://localhost/api/datasources/uid/foo42",
            json=PROMETHEUS_DATASOURCE,
        )

        result = self.grafana.datasource.update_datasource_by_uid("foo42", PROMETHEUS_DATASOURCE)
        self.assertEqual(result["type"], "prometheus")

    @requests_mock.Mocker()
    def test_delete_datasource_by_id(self, m):
        m.delete("http://localhost/api/datasources/42", json={"message": "Data source deleted"})

        result = self.grafana.datasource.delete_datasource_by_id(42)
        self.assertEqual(result, {"message": "Data source deleted"})

    @requests_mock.Mocker()
    def test_delete_datasource_by_uid(self, m):
        m.delete("http://localhost/api/datasources/uid/h8KkCLt7z", json={"message": "Data source deleted"})

        result = self.grafana.datasource.delete_datasource_by_uid("h8KkCLt7z")
        self.assertEqual(result, {"message": "Data source deleted"})

    @requests_mock.Mocker()
    def test_delete_datasource_by_name(self, m):
        m.delete("http://localhost/api/datasources/name/Prometheus", json={"message": "Data source deleted"})

        result = self.grafana.datasource.delete_datasource_by_name("Prometheus")
        self.assertEqual(result, {"message": "Data source deleted"})

    @requests_mock.Mocker()
    def test_enable_datasource_permissions(self, m):
        m.post(
            "http://localhost/api/datasources/42/enable-permissions", json={"message": "Datasource permissions enabled"}
        )

        result = self.grafana.datasource.enable_datasource_permissions(42)
        self.assertEqual(result, {"message": "Datasource permissions enabled"})

    @requests_mock.Mocker()
    def test_disable_datasource_permissions(self, m):
        m.post(
            "http://localhost/api/datasources/42/disable-permissions",
            json={"message": "Datasource permissions disabled"},
        )

        result = self.grafana.datasource.disable_datasource_permissions(42)
        self.assertEqual(result, {"message": "Datasource permissions disabled"})

    @requests_mock.Mocker()
    def test_get_datasource_permissions(self, m):
        m.get(
            "http://localhost/api/datasources/42/permissions",
            json=PERMISSION_DATASOURCE,
        )

        result = self.grafana.datasource.get_datasource_permissions(42)
        self.assertEqual(result["datasourceId"], 42)

    @requests_mock.Mocker()
    def test_add_datasource_permissions(self, m):
        m.post("http://localhost/api/datasources/42/permissions", json={"message": "Datasource permission added"})

        result = self.grafana.datasource.add_datasource_permissions(42, {"userId": 1, "permission": 1})
        self.assertEqual(result, {"message": "Datasource permission added"})

    @requests_mock.Mocker()
    def test_remove_datasource_permissions(self, m):
        m.delete("http://localhost/api/datasources/42/permissions/1", json={"message": "Datasource permission removed"})

        result = self.grafana.datasource.remove_datasource_permissions(42, 1)
        self.assertEqual(result, {"message": "Datasource permission removed"})

    @requests_mock.Mocker()
    def test_find_datasource(self, m):
        m.get(
            "http://localhost/api/datasources/name/Prometheus",
            json=PROMETHEUS_DATASOURCE,
        )

        result = self.grafana.datasource.find_datasource("Prometheus")
        self.assertEqual(result["type"], "prometheus")

    @requests_mock.Mocker()
    def test_find_datasource_not_existing(self, m):
        m.get(
            "http://localhost/api/datasources/name/it_doesnot_exist",
            json={"message": "Data source not found"},
            status_code=400,
        )

        with self.assertRaises(GrafanaBadInputError):
            self.grafana.datasource.find_datasource("it_doesnot_exist")

    @requests_mock.Mocker()
    def test_get_datasource_id_by_name(self, m):
        m.get("http://localhost/api/datasources/id/Prometheus", json={"id": 42})

        result = self.grafana.datasource.get_datasource_id_by_name("Prometheus")
        self.assertEqual(result["id"], 42)

    @requests_mock.Mocker()
    def test_list_datasources(self, m):
        m.get(
            "http://localhost/api/datasources",
            json=[PROMETHEUS_DATASOURCE],
        )

        result = self.grafana.datasource.list_datasources()
        self.assertEqual(result[0]["type"], "prometheus")
        self.assertEqual(len(result), 1)

    @requests_mock.Mocker()
    def test_get_datasource_proxy_data_query(self, m):
        # http://localhost:3000/api/datasources/proxy/1/api/v1/query?query=up%7binstance%3d%22localhost:9090%22%7d&time=1644164339
        m.post(
            "http://localhost/api/datasources/proxy/1/api/v1/query",
            json=PROMETHEUS_DATA_RESPONSE,
        )
        result = self.grafana.datasource.get_datasource_proxy_data(
            1,  # datasource_id
            query_type="query",
            expr='up{instance="localhost:9090"}',
            time=1644164339,
        )
        self.assertEqual(result["status"], "success")
        self.assertEqual(result["data"]["result"][0]["metric"]["job"], "prometheus")
        self.assertEqual(len(result["data"]["result"][0]["values"]), 6)

    @requests_mock.Mocker()
    def test_get_datasource_proxy_data_query_range(self, m):
        # http://localhost:3000/api/datasources/proxy/1/api/v1/query_range?query=up%7binstance%3d%22localhost:9090%22%7d&start=1644164339&end=1644164639&step=60
        m.post(
            "http://localhost/api/datasources/proxy/1/api/v1/query_range",
            json=PROMETHEUS_DATA_RESPONSE,
        )
        result = self.grafana.datasource.get_datasource_proxy_data(
            1,  # datasource_id
            query_type="query_range",
            expr='up{instance="localhost:9090"}',
            start=1644164339,
            end=1644164639,
            step=60,
        )
        self.assertEqual(result["status"], "success")
        self.assertEqual(result["data"]["result"][0]["metric"]["job"], "prometheus")
        self.assertEqual(len(result["data"]["result"][0]["values"]), 6)

    @requests_mock.Mocker()
    def test_get_datasource_proxy_data_failure(self, m):
        m.post(
            "http://localhost/api/datasources/proxy/1/api/v1/query",
            json=PROMETHEUS_DATA_RESPONSE,
        )
        with self.assertRaises(KeyError) as ctx:
            self.grafana.datasource.get_datasource_proxy_data(
                1,  # datasource_id
                query_type="foobar",
                expr='up{instance="localhost:9090"}',
                time=1644164339,
            )
        self.assertEqual(str(ctx.exception), "'Unknown or invalid query type: foobar'")

    @requests_mock.Mocker()
    def test_series(self, m):
        # TODO: Not sure if this emulates the right payloads.
        #       It has been copied from the other test functions above.
        m.post(
            "http://localhost/api/datasources/proxy/1/api/v1/series",
            json=PROMETHEUS_DATA_RESPONSE,
        )
        result = self.grafana.datasource.series(
            datasource_id=1,
            match="foo",
            start=1644164339,
            end=1644164639,
        )
        self.assertEqual(result["status"], "success")
        self.assertEqual(result["data"]["result"][0]["metric"]["job"], "prometheus")
        self.assertEqual(len(result["data"]["result"][0]["values"]), 6)

    @requests_mock.Mocker()
    def test_query_with_datasource_prometheus(self, m):
        m.post(
            "http://localhost/api/ds/query",
            json=DATAFRAME_RESPONSE_HEALTH_PROMETHEUS,
        )
        datasource = PROMETHEUS_DATASOURCE.copy()
        response = self.grafana.datasource.smartquery(datasource, "1+1")
        self.assertEqual(response, DATAFRAME_RESPONSE_HEALTH_PROMETHEUS)

    @requests_mock.Mocker()
    def test_query_with_datasource_influxdb_influxql(self, m):
        m.post(
            "http://localhost/api/datasources/proxy/43/query",
            json={},
        )
        datasource = INFLUXDB1_DATASOURCE.copy()
        _ = self.grafana.datasource.smartquery(datasource, "SHOW RETENTION POLICIES on _internal", store="foobar")
        # TODO: No response payload yet.

    @requests_mock.Mocker()
    def test_query_with_datasource_elasticsearch(self, m):
        m.get(
            "http://localhost/api/datasources/proxy/44/bazqux/_mapping",
            json={},
        )
        datasource = ELASTICSEARCH_DATASOURCE.copy()
        _ = self.grafana.datasource.smartquery(
            datasource, "url:///datasources/proxy/44/bazqux/_mapping", store="bazqux"
        )
        # TODO: No response payload yet.

    @requests_mock.Mocker()
    def test_query_with_datasource_identifier(self, m):
        m.get(
            "http://localhost/api/datasources/uid/h8KkCLt7z",
            json=PROMETHEUS_DATASOURCE,
        )
        m.post(
            "http://localhost/api/ds/query",
            json=DATAFRAME_RESPONSE_HEALTH_PROMETHEUS,
        )
        response = self.grafana.datasource.smartquery(DatasourceIdentifier(uid="h8KkCLt7z"), "1+1")
        self.assertEqual(response, DATAFRAME_RESPONSE_HEALTH_PROMETHEUS)

    def test_query_unknown_access_type_failure(self):
        datasource = PROMETHEUS_DATASOURCE.copy()
        datasource["access"] = "__UNKNOWN__"
        self.assertRaises(NotImplementedError, lambda: self.grafana.datasource.smartquery(datasource, expression="1+1"))

    def test_query_empty_expression_failure(self):
        datasource = PROMETHEUS_DATASOURCE.copy()
        self.assertRaises(ValueError, lambda: self.grafana.datasource.smartquery(datasource, expression=None))

    @patch("grafana_client.client.GrafanaClient.__getattr__")
    def test_query_client_error_failure(self, mock_get):

        mock_get.return_value = Mock()
        mock_get.return_value.side_effect = GrafanaClientError(status_code=400, response={}, message="Something failed")

        datasource = PROMETHEUS_DATASOURCE.copy()
        self.assertRaises(GrafanaClientError, lambda: self.grafana.datasource.smartquery(datasource, expression="1+1"))

    @patch("grafana_client.client.GrafanaClient.__getattr__")
    def test_query_server_error_failure(self, mock_get):

        mock_get.return_value = Mock()
        mock_get.return_value.side_effect = GrafanaServerError(
            status_code=500, response={}, message="Something serious failed"
        )

        datasource = PROMETHEUS_DATASOURCE.copy()
        self.assertRaises(GrafanaServerError, lambda: self.grafana.datasource.smartquery(datasource, expression="1+1"))
